<?php

/**
 * @author: Lukasz Sosna
 * @copyright: 2017-11-29
 * @e-mail: tree@interia.pl
 * @www: http://en.lukasz.sos.pl/
 * @version: 1.0.0
 */
 
namespace app\controllers;

use Yii;
use app\models\Blogadmin;
use yii\data\ActiveDataProvider;
use yii\web\Controller;
use yii\web\NotFoundHttpException;
use yii\filters\VerbFilter;

class BlogadminController extends Controller
{
    public $layout = 'admin';
    
    public function beforeAction($action) 
    {
        $session = Yii::$app->session;

        if($session['yii_user_id'] != "")
        {
            if($session['yii_user_root'] != "y")
            {
                return $this->redirect(['/right']);
            }
        }
        else
        {
            return $this->redirect(['/login']);
        }
        
        return parent::beforeAction($action);
    }
    
    public function behaviors()
    {
        return [
            'verbs' => [
                'class' => VerbFilter::className(),
                'actions' => [
                    'delete' => ['POST'],
                ],
            ],
        ];
    }

    public function actionIndex()
    {
        Yii::$app->OtherFunctionsComponent->WriteLog(Yii::t('app', 'log_browse_blog'));
        
        $dataProvider = new ActiveDataProvider([
            'query' => Blogadmin::find(),
        ]);

        return $this->render('index', [
            'dataProvider' => $dataProvider,
        ]);
    }

    public function actionView($id)
    {
        Yii::$app->OtherFunctionsComponent->WriteLog(Yii::t('app', 'log_browse_one_entry_blog'));
        
        return $this->render('view', [
            'model' => $this->findModel($id),
        ]);
    }

    public function actionCreate()
    {
        $model = new Blogadmin();
        
        $ItemToList = $model->CreateCategoryList();

        if ($model->load(Yii::$app->request->post()) && $model->save()) 
        {
            Yii::$app->OtherFunctionsComponent->WriteLog(Yii::t('app', 'log_blog_add'));
            
            return $this->redirect(['view', 'id' => $model->blog_id]);
        } else {
            return $this->render('create', [
                'model' => $model, 'ItemToList' => $ItemToList,
            ]);
        }
    }

    public function actionUpdate($id)
    {
        $model = new Blogadmin();
        
        $ItemToList = $model->CreateCategoryList();
        
        $model = $this->findModel($id);

        if ($model->load(Yii::$app->request->post()) && $model->save())
        {
            Yii::$app->OtherFunctionsComponent->WriteLog(Yii::t('app', 'log_blog_updated'));
            
            return $this->redirect(['view', 'id' => $model->blog_id]);
        } else {
            return $this->render('update', [
                'model' => $model, 'ItemToList' => $ItemToList,
            ]);
        }
    }

    public function actionDelete($id)
    {
        $this->findModel($id)->delete();
        
        Yii::$app->OtherFunctionsComponent->WriteLog(Yii::t('app', 'log_blog_delete'));
        
        return $this->redirect(['index']);
    }

    protected function findModel($id)
    {
        if (($model = Blogadmin::findOne($id)) !== null) {
            return $model;
        } else {
            throw new NotFoundHttpException(Yii::t('app', 'page_does_not_exists'));
        }
    }
}
