<?php

/**
 * @author: Lukasz Sosna
 * @copyright: 2017-11-29
 * @e-mail: tree@interia.pl
 * @www: http://en.lukasz.sos.pl/
 * @version: 1.0.0
 */
 
namespace app\controllers;

use Yii;
use yii\web\Controller;
use app\models\User;
use app\models\Error404;

class UserController extends Controller
{
    public function actions()
    {
        return [
            'captcha' => [
                'class' => 'yii\captcha\CaptchaAction',
                'fixedVerifyCode' => YII_ENV_TEST ? 'testme' : null,
            ],
        ];
    }
    
    public function actionError()
    {
        $model = new Error404();
        $model->AddError404();
        
        return $this->render('error');
    }
    
    public function actionIndex()
    {
        return $this->render('index');
    }

    public function actionLogin()
    {
        $session = Yii::$app->session;

        if($session['yii_user_id'] == "")
        {
            $model = new User(['scenario' => User::SCENARIO_LOGIN]);
            
            if ($model->load(Yii::$app->request->post()) && $model->validate()) 
            {
                $this->goHome();
              } 
            else 
            {
                return $this->render('login', ['model' => $model]);
            }
        }
        else
        {
            $this->goHome();
        }
    }
    
    public function actionLogout()
    {
        $session = Yii::$app->session;
        $session['yii_user_id'] = '';
        $session['yii_user_email'] = '';
        $session['yii_user_root'] = '';
        $this->goHome();
    }

    public function actionRempass()
    {
        $model = new User(['scenario' => User::SCENARIO_REMIND]);
        
        if ($model->load(Yii::$app->request->post()) && $model->validate()) 
        {
            //
            // Send e-mail
            //
            $UserRegisterIp = $_SERVER['REMOTE_ADDR'];
            
            Yii::$app->mailer->compose()
            ->setFrom(array(Yii::$app->params['adminEmail'] => Yii::$app->params['adminName']))
            ->setTo($model->user_email)
            ->setSubject(Yii::t('app', 'user_remind_pass_title').Yii::$app->params['pageTitle'])
            ->setTextBody(
            Yii::t('app', 'user_remind_pass_body', [
            'page_title' => Yii::$app->params['pageTitle'],
            'user_ip' => $UserRegisterIp,
            'administrator' => Yii::$app->params['adminName'],
            'page_link' => Yii::$app->params['pageUrl'].'remind-password-set/'.$model->user_id.'/'.$model->password_hash1.'/'.$model->password_hash2
            ])
            )
            ->send();
            
            return $this->render('rempass-success', ['model' => $model]);
        }
        else
        {
            return $this->render('rempass', ['model' => $model]);
        }
    }
    
    public function actionRempassset($UserId,$UserHash1,$UserHash2)
    {
        $model = new User();
        
        $model->SetNewPassword($UserId,$UserHash1,$UserHash2);
        $ReturnRemPass = $model->user_password;
        
        $ViewPassChanged = false;
        
        if($ReturnRemPass)
        {
            //
            // Send e-mail
            //
            $UserRegisterIp = $_SERVER['REMOTE_ADDR'];
            
            Yii::$app->mailer->compose()
            ->setFrom(array(Yii::$app->params['adminEmail'] => Yii::$app->params['adminName']))
            ->setTo($model->user_email)
            ->setSubject(Yii::t('app', 'user_new_password').Yii::$app->params['pageTitle'])
            ->setTextBody(
            Yii::t('app', 'user_new_password_body', [
            'page_title' => Yii::$app->params['pageTitle'],
            'user_ip' => $UserRegisterIp,
            'administrator' => Yii::$app->params['adminName'],
            'user_password' => $ReturnRemPass,
            ]
            ))
            ->send();
            
            $ViewPassChanged = true;
        }
        
        return $this->render('rempass-set', ['model' => $model, 'ViewPassChanged' => $ViewPassChanged]);
    }
    
    public function actionActivate($UserId,$UserKey)
    {       
        $model = new User(['scenario' => User::SCENARIO_ACTIVATE]);
        
        $MessageGood = $model->ActivateUser($UserId,$UserKey);
        
        return $this->render('activate', ['model' => $model, 'MessageGood' => $MessageGood]);
    }
    
    public function actionRegister()
    {       
        $model = new User(['scenario' => User::SCENARIO_REGISTER]);
        
        if ($model->load(Yii::$app->request->post()) && $model->validate()) 
        {
            $UserData = $model->RegisterUser();
            
            $UserRegisterIp = $_SERVER['REMOTE_ADDR'];
            
            Yii::$app->mailer->compose()
            ->setFrom(array(Yii::$app->params['adminEmail'] => Yii::$app->params['adminName']))
            ->setTo($model->user_email)
            ->setSubject(Yii::t('app', 'user_actvate_account').Yii::$app->params['pageTitle'])
            ->setTextBody(
            Yii::t('app', 'user_actvate_account_body', [
            'page_title' => Yii::$app->params['pageTitle'],
            'user_ip' => $UserRegisterIp,
            'administrator' => Yii::$app->params['adminName'],
            'page_link' => Yii::$app->params['pageUrl'].'activate/'.$UserData['user_id'].'/'.$UserData['user_key'],
            ]
            ))
            ->send();
            
            return $this->render('register-success', ['model' => $model]);
        }
        else
        {
            $model->user_www = str_replace('&amp;', '&', $model->user_www);
            
            return $this->render('register', ['model' => $model]);
        }
    }
     
    public function actionProfile()
    {
        $dataChanged = false;
        
        $session = Yii::$app->session;
        
        if($session['yii_user_id'] != "")
        {
            $request = Yii::$app->request;
            $post = $request->post();
            
            $model = new User(['scenario' => User::SCENARIO_UPDATEPROFILE]);
            
            $DataToProfile = $model->SelectProfile();
                
            $model->user_namesurname = $DataToProfile['user_namesurname'];
            $model->user_phone = $DataToProfile['user_phone'];
            $model->user_www = $DataToProfile['user_www'];
            $model->user_about = $DataToProfile['user_about'];
            
            $model->user_www = str_replace('&amp;', '&', $model->user_www);
                
            if ($model->load(Yii::$app->request->post()) && $model->validate()) 
            {
                $model->UpdateProfile();
                
                $dataChanged = true;
                
                $model->user_namesurname = $post['User']['user_namesurname'];
                $model->user_phone = $post['User']['user_phone'];
                $model->user_www = $post['User']['user_www'];
                $model->user_about = $post['User']['user_about'];
            }
            
            return $this->render('profile', ['model' => $model, 'dataChanged' => $dataChanged]);
        }
        else
        {
            $this->goHome();
        }
    }
    
    public function actionChangepassword()
    {
        $passwordChanged = false;
        
        $session = Yii::$app->session;

        if($session['yii_user_id'] != "")
        {
            $model = new User(['scenario' => User::SCENARIO_CHANGEPASSWORD]);
            
            if ($model->load(Yii::$app->request->post()) && $model->validate()) 
            {
                $model->UpdatePassword();
                
                $model->user_password = '';
                $model->user_password2 = '';
                $model->user_password3 = '';
                
                $passwordChanged = true;
            }
            
            return $this->render('changepassword', ['model' => $model, 'passwordChanged' => $passwordChanged]);
        }
        else
        {
            $this->goHome();
        }
    }
    
    public function actionRight()
    {
        return $this->render('right');
    }
}
